#ifndef _CHRONOPTICS_TOF_KEA_CAMERA_H_
#define _CHRONOPTICS_TOF_KEA_CAMERA_H_

#include <chronoptics/tof/camera.h>
#include <chronoptics/tof/processing_config.h>
#include <chronoptics/tof/camera_config.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The main interface to the kea camera
 */
typedef struct tof_kea_camera* tof_kea_camera_t;

/** Destruct tof_kea_camera */
TOF_EXPORT void tof_kea_camera_delete(tof_kea_camera_t ptr);

/** Construct the kea camera
 * @param ptr Pointer to class
 * @param processing_config The processing config
 * @param serial The serial number of the camera to connect to, if the camera is
 * available over both usb and gige it will be connected over usb. If an empty
 * string is supplied the camera will connect to the first usb camera, if no usb
 * camera is connected it will connect to the first gige camera that replies
 * (this reply time is random, so you might connect to a different camera each
 * time if you've got multiple cameras connected to your network). If no camera
 * is found with the given serial number a nullptr and error is returned.
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_kea_camera_t tof_kea_camera_new(const tof_processing_config_t processing_config, const char* serial, tof_error_t *error);

/** Check whether this kea camera can do on camera processing
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return On camera processing capable
 */
TOF_EXPORT bool tof_kea_camera_on_camera_processing_capable(const tof_kea_camera_t ptr, tof_error_t *error);

/** Get on camera processing
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return True on camera processing, false on host processing
 */
TOF_EXPORT bool tof_kea_camera_get_on_camera_processing(const tof_kea_camera_t ptr, tof_error_t *error);

/** Set on camera processing
 * @param ptr Pointer to class
 * @param on_camera_processing True on camera processing, false on host
 * processing
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_on_camera_processing(tof_kea_camera_t ptr, bool on_camera_processing, tof_error_t *error);

/** Get delay between network packets
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The delay, depending on the camera software it is in nanoseconds or
 * in cpu instruction
 */
TOF_EXPORT uint32_t tof_kea_camera_get_delay(const tof_kea_camera_t ptr, tof_error_t *error);

/** Set delay between network packets
 * @param ptr Pointer to class
 * @param delay The delay, depending on the camera software it is in nanoseconds
 * or in cpu instruction
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_delay(tof_kea_camera_t ptr, uint32_t delay, tof_error_t *error);

/** Get maximum size of each network packet transmitted
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Packet size in bytes
 */
TOF_EXPORT uint16_t tof_kea_camera_get_packet_size(const tof_kea_camera_t ptr, tof_error_t *error);

/** Set maximum size of each network packet transmitted
 * @param ptr Pointer to class
 * @param size Packet size in bytes
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_packet_size(tof_kea_camera_t ptr, uint16_t size, tof_error_t *error);

/** Set the camera config
 * @param ptr Pointer to class
 * @param camera_config Camera config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_camera_config(tof_kea_camera_t ptr, const tof_camera_config_t camera_config, tof_error_t *error);

/** Set both the camera and processing config
 * @param ptr Pointer to class
 * @param camera_config Camera config
 * @param processing_config Processing config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_kea_camera_set_configurations(tof_kea_camera_t ptr, const tof_camera_config_t camera_config, const tof_processing_config_t processing_config, tof_error_t *error);

/** Get the tof library version running on the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Version
 */
TOF_EXPORT const char* tof_kea_camera_version(tof_kea_camera_t ptr, tof_error_t *error);

/** Simple class containing information about found kea cameras
 */
typedef struct tof_discovered_kea* tof_discovered_kea_t;

/** Destruct tof_discovered_kea */
TOF_EXPORT void tof_discovered_kea_delete(tof_discovered_kea_t ptr);

/** The serial
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Serial
 */
TOF_EXPORT const char* tof_discovered_kea_serial(const tof_discovered_kea_t ptr, tof_error_t *error);

/** Additional information about the discovered camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Additional information
 */
TOF_EXPORT const char* tof_discovered_kea_info(const tof_discovered_kea_t ptr, tof_error_t *error);

/** Discover all available kea cameras
 * @param available_kea_cameras All available kea cameras
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_discover_kea_cameras(tof_discovered_kea_t* available_kea_cameras, size_t capacity, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
