#ifndef _CHRONOPTICS_TOF_USB_INTERFACE_HPP_
#define _CHRONOPTICS_TOF_USB_INTERFACE_HPP_

#include <chronoptics/tof/usb_interface.h>

#include <chronoptics/tof/camera_config.hpp>
#include <chronoptics/tof/calibration.hpp>
#include <chronoptics/tof/data.hpp>
#include <chronoptics/tof/gige_interface.hpp>

namespace chronoptics {
namespace tof {

/** USB device
*/
class UsbDevice : public detail::Base<tof_usb_device, tof_usb_device_delete> {
 public:
  /** Construct from pointer */
  UsbDevice(tof_usb_device_t ptr = nullptr) {
    this->ptr_ = ptr;
  }

  /** Get serial corresponding with USB device
   * @return Serial
   */
  const char* serial() {
    return tof_usb_device_serial(this->ptr_, TOF_ERROR_HANDLER{});
  }

};

/** Connect to a camera directly using the USB interface
*/
class UsbInterface : public detail::Base<tof_usb_interface, tof_usb_interface_delete> {
 public:
  /** Construct from pointer */
  UsbInterface(tof_usb_interface_t ptr = nullptr) {
    this->ptr_ = ptr;
  }

  /** Create USB camera from a usb device
   * @param usb_device USB device
   */
  UsbInterface(UsbDevice &usb_device) {
    this->ptr_ = tof_usb_interface_new(*reinterpret_cast<tof_usb_device_t*>(&usb_device), TOF_ERROR_HANDLER{});
  }

  /** Check whether a connection is still maintained with the camera
   * @return Is connected
   */
  bool is_connected() const {
    return tof_usb_interface_is_connected(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Tests the connection speed of the USB port. This is useful to check
   * whether a hub or USB port works well. The maximum speed that the camera is
   * able to transmit is ~3.0 Gbps.
   * @return Connection speed in Gpbs (bits)
   */
  double test_stream_speed() {
    return tof_usb_interface_test_stream_speed(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Download camera configuration from the camera
   * @return Camera configuration
   */
  CameraConfig download_configuration() {
    CameraConfig new_camera_config(static_cast<tof_camera_config_t>(nullptr));
    auto ptr = reinterpret_cast<tof_camera_config_t*>(&new_camera_config);
    *ptr = tof_usb_interface_download_configuration(this->ptr_, TOF_ERROR_HANDLER{});
    return new_camera_config;
  }

  /** Download calibration from the camera
   * @return Calibration
   */
  Calibration download_calibration() {
    Calibration new_calibration(static_cast<tof_calibration_t>(nullptr));
    auto ptr = reinterpret_cast<tof_calibration_t*>(&new_calibration);
    *ptr = tof_usb_interface_download_calibration(this->ptr_, TOF_ERROR_HANDLER{});
    return new_calibration;
  }

  /** Upload camera configuration to the camera
   * @param config Camera configuration
   */
  void upload_configuration(const CameraConfig &config) {
    return tof_usb_interface_upload_configuration(this->ptr_, *reinterpret_cast<const tof_camera_config_t*>(&config), TOF_ERROR_HANDLER{});
  }

  /** Check whether this camera is capable of streaming the specified type
   * @param stream Depth stream type
   * @return Is capable
   */
  bool depth_stream_capable(DepthStreamType stream) const {
    return tof_usb_interface_depth_stream_capable(this->ptr_, static_cast<tof_depth_stream_type>(stream), TOF_ERROR_HANDLER{});
  }

  /** Start the depth stream
   * @param stream Depth stream type
   */
  void start_depth_stream(DepthStreamType stream) {
    return tof_usb_interface_start_depth_stream(this->ptr_, static_cast<tof_depth_stream_type>(stream), TOF_ERROR_HANDLER{});
  }

  /** Stop the depth stream
   */
  void stop_depth_stream() {
    return tof_usb_interface_stop_depth_stream(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Returns true if a depth frame is available
   * @return Depth frame available
   */
  bool has_depth_data() const {
    return tof_usb_interface_has_depth_data(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Get a depth frame
   * @return Depth frame
   */
  Data get_depth_data() {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_usb_interface_get_depth_data(this->ptr_, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Check whether camera supports an image stream
   * @return Camera supports an image stream
   */
  bool image_stream_capable() const {
    return tof_usb_interface_image_stream_capable(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Start image stream
   */
  void start_image_stream() {
    return tof_usb_interface_start_image_stream(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Stop image stream
   */
  void stop_image_stream() {
    return tof_usb_interface_stop_image_stream(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Check whether image frame is available
   * @return Image frame available
   */
  bool has_image_data() const {
    return tof_usb_interface_has_image_data(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Get image frame
   * @return Image data
   */
  Data get_image_data() {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_usb_interface_get_image_data(this->ptr_, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Get the image frame into the supplied pointer
   * @param pointer The pointer to the data to write into
   * @param capacity The amount of data the pointer can hold
   * @param callback Callback that will be called when pointer is no longer in
   * use
   * @return Image data
   */
  Data get_image_data_into_pointer(uint8_t* pointer, size_t capacity, user_pointer_destructed_fn &callback) {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_usb_interface_get_image_data_into_pointer(this->ptr_, pointer, capacity, user_pointer_destructed_cb_handler, &callback, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Get the image frame into the supplied pointer
   * @param pointer The pointer to the data to write into
   * @param capacity The amount of data the pointer can hold
   * @param callback Callback that will be called when pointer is no longer in
   * use
   * @param callback_user_data User data that will be passed back when the function pointer is called
   * @return Image data
   */
  Data get_image_data_into_pointer(uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data) {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_usb_interface_get_image_data_into_pointer(this->ptr_, pointer, capacity, callback, callback_user_data, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Get the capacity for amount of user pointers that can be registered
   * @return User pointer capacity
   */
  size_t get_user_pointer_capacity() const {
    return tof_usb_interface_get_user_pointer_capacity(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Set the capacity for amount of user pointers that can be registered
   * @param capacity User pointer capacity
   */
  void set_user_pointer_capacity(size_t capacity) {
    return tof_usb_interface_set_user_pointer_capacity(this->ptr_, capacity, TOF_ERROR_HANDLER{});
  }

  /** Add a pointer that will be filled with depth data. This method allows you
   * to fill data straight into your own data structure without any additional
   * copying.
   * @param pointer The pointer to the data to write into
   * @param capacity The amount of data the pointer can hold
   * @param callback Callback that will be called when pointer is no longer in
   * use
   */
  void add_depth_user_pointer(uint8_t* pointer, size_t capacity, user_pointer_destructed_fn &callback) {
    return tof_usb_interface_add_depth_user_pointer(this->ptr_, pointer, capacity, user_pointer_destructed_cb_handler, &callback, TOF_ERROR_HANDLER{});
  }

  /** Add a pointer that will be filled with depth data. This method allows you
   * to fill data straight into your own data structure without any additional
   * copying.
   * @param pointer The pointer to the data to write into
   * @param capacity The amount of data the pointer can hold
   * @param callback Callback that will be called when pointer is no longer in
   * use
   * @param callback_user_data User data that will be passed back when the function pointer is called
   */
  void add_depth_user_pointer(uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data) {
    return tof_usb_interface_add_depth_user_pointer(this->ptr_, pointer, capacity, callback, callback_user_data, TOF_ERROR_HANDLER{});
  }

  /** Get the tof library version running on the camera
   * @return Version
   */
  const char* version() {
    return tof_usb_interface_version(this->ptr_, TOF_ERROR_HANDLER{});
  }

};

/** Discover all connected chronoptics usb cameras
 * @return The usb devices
 */
inline std::vector<UsbDevice> usb_device_discover() {
  size_t size = tof_usb_device_discover(nullptr, 0, TOF_ERROR_HANDLER{});
  std::vector<UsbDevice> vec;
  vec.reserve(size);
  for (size_t i = 0; i < size; i++)
    vec.emplace_back(static_cast<tof_usb_device_t>(nullptr));
  auto data = reinterpret_cast<tof_usb_device_t*>(vec.data());
  size = tof_usb_device_discover(data, vec.size(), TOF_ERROR_HANDLER{});
  return vec;
}

/** Discover one chronoptics usb camera
 * @return A USB device
 */
inline UsbDevice usb_device_discover_one() {
  UsbDevice new_usb_device(static_cast<tof_usb_device_t>(nullptr));
  auto ptr = reinterpret_cast<tof_usb_device_t*>(&new_usb_device);
  *ptr = tof_usb_device_discover_one(TOF_ERROR_HANDLER{});
  return new_usb_device;
}

/** Discover usb camera with the given serial
 * @param serial Serial of the camera to find
 * @return A USB device
 */
inline UsbDevice usb_device_find(StringView serial) {
  UsbDevice new_usb_device(static_cast<tof_usb_device_t>(nullptr));
  auto ptr = reinterpret_cast<tof_usb_device_t*>(&new_usb_device);
  *ptr = tof_usb_device_find(serial, TOF_ERROR_HANDLER{});
  return new_usb_device;
}

} // tof
} // chronoptics

#endif
