classdef CsfCamera < tof.Camera
  % CsfCamera
  %   The csf camera class contains all the logic to generate a stream of depth
  %   data from a csf file
  %
  % Methods:
  %   getFramesAt - Get frames at the specified position
  %   getCurrentFrame - Get the location (count) of the current frame
  %   getFrameNumber - Get the amount of frame lists this csf file can
  %                    open/generate
  %   getFps - Get the FPS If the FPS is greater than 0 the time between
  %            subsequent function calls to get_frames or get_frames_at will be
  %            limited to the FPS This is done by sleeping on the next call to
  %            get_frames() if the time between calls is smaller than the time
  %            duration dictated by the fps.

  %   setFps - Set the FPS If the FPS is greater than 0 the time between
  %            subsequent function calls to get_frames or get_frames_at will be
  %            limited to the FPS This is done by sleeping on the next call to
  %            get_frames() if the time between calls is smaller than the time
  %            duration dictated by the fps.

  %

  methods(Access=public)
    function obj = CsfCamera(varargin)
      % CsfCamera
      %   Construct new csf_camera class
      %
      % ----------
      %
      %   Construct a csf camera from a csf file
      %
      % Inputs:
      %   1 - ProcessingConfig - A processing config
      %   2 - string - File location of the csf file
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        ptr = varargin{1};
      elseif nargin == 2 && isa(varargin{1}, 'tof.ProcessingConfig') && (isStringScalar(varargin{2}) || ischar(varargin{2}))
        ptr = tof.mex('tof_csf_camera_new', varargin{1}.get_object_ptr(), varargin{2});
      else
        error("Can't construct empty object");
      end

      obj@tof.Camera(ptr);

    end

    function delete(~)
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = getFramesAt(obj, frame_nr)
      % getFramesAt
      %   Get frames at the specified position
      %
      % Inputs:
      %   frame_nr - numeric - Which set of frames to process
      %
      % Output:
      %   ret_val - vector[Data] - Data frames
      %

      ret_val = tof.mex('tof_csf_camera_get_frames_at', obj.mPtr, frame_nr);
      ret_val = arrayfun(@(vec_obj)tof.Data(vec_obj), ret_val);
    end

    function ret_val = getCurrentFrame(obj)
      % getCurrentFrame
      %   Get the location (count) of the current frame
      %
      % Output:
      %   ret_val - numeric - The currenc frame location
      %

      ret_val = tof.mex('tof_csf_camera_get_current_frame', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getFrameNumber(obj)
      % getFrameNumber
      %   Get the amount of frame lists this csf file can open/generate
      %
      % Output:
      %   ret_val - numeric - The number of frames in the csf file
      %

      ret_val = tof.mex('tof_csf_camera_get_frame_number', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getFps(obj)
      % getFps
      %   Get the FPS If the FPS is greater than 0 the time between subsequent
      %   function calls to get_frames or get_frames_at will be limited to the
      %   FPS This is done by sleeping on the next call to get_frames() if the
      %   time between calls is smaller than the time duration dictated by the
      %   fps.

      %
      % Output:
      %   ret_val - numeric - The frames per second
      %

      ret_val = tof.mex('tof_csf_camera_get_fps', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setFps(obj, fps)
      % setFps
      %   Set the FPS If the FPS is greater than 0 the time between subsequent
      %   function calls to get_frames or get_frames_at will be limited to the
      %   FPS This is done by sleeping on the next call to get_frames() if the
      %   time between calls is smaller than the time duration dictated by the
      %   fps.

      %
      % Inputs:
      %   fps - numeric - The frames per second
      %

      tof.mex('tof_csf_camera_set_fps', obj.mPtr, fps);
    end

  end
end
