classdef UserConfig < handle
  % UserConfig
  %   Simple interface for camera configuration generation
  %
  % Methods:
  %   toCameraConfig - Convert to a camera configuration
  %   getEnvironment - Get the camera operating environment
  %   setEnvironment - Set the camera operating environment
  %   getMaxDistance - Get the maximum distance photons reflect from
  %   setMaxDistance - Set the maximum distance photons reflect from
  %   getFps - Get the depth frame rate
  %   setFps - Set the depth frame rate
  %   getLensFlare - Get if lens flare is expected in the scene
  %   setLensFlare - Set if lens flare is expected in the scene
  %   getTranslucent - Get if there are translucent objects in the scene
  %   setTranslucent - Set if there are translucent objects in the scene
  %   getHdr - Get high dynamic range enabled. Enable this when pixel saturation
  %            occurs. This will only happen for retroreflectors or reflective
  %            objects close (<20cm) to the camera
  %   setHdr - Set high dynamic range enabled. Enable this when pixel saturation
  %            occurs. This will only happen for retroreflectors or reflective
  %            objects close (<20cm) to the camera
  %   getIntegrationTime - Get the integration time
  %   setIntegrationTime - Set the integration time
  %   getStrategy - Get what the configuration is being optimized for
  %   setStrategy - Set what the configuration is being optimized for
  %   getChannel - Get The camera channel, increment to support multi-camera
  %                usage
  %   setChannel - Set The camera channel, increment to support multi-camera
  %                usage
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = UserConfig(varargin)
      % UserConfig
      %   Construct new user_config class
      %
      % ----------
      %
      %   Create default user config
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      else
        obj.mPtr = tof.mex('tof_user_config_new_default');
      end
    end

    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_user_config_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = toCameraConfig(obj, camera)
      % toCameraConfig
      %   Convert to a camera configuration
      %
      % Inputs:
      %   camera - Camera - The camera
      %
      % Output:
      %   ret_val - CameraConfig - The camera configuration
      %

      ret_val = tof.mex('tof_user_config_to_camera_config', obj.mPtr, camera.get_object_ptr());
      ret_val = tof.CameraConfig(ret_val);
    end

    function ret_val = getEnvironment(obj)
      % getEnvironment
      %   Get the camera operating environment
      %
      % Output:
      %   ret_val - ImagingEnvironment - Imaging environment
      %

      ret_val = tof.mex('tof_user_config_get_environment', obj.mPtr);
    end

    function setEnvironment(obj, imaging_environment)
      % setEnvironment
      %   Set the camera operating environment
      %
      % Inputs:
      %   imaging_environment - ImagingEnvironment - Imaging environment
      %

      tof.mex('tof_user_config_set_environment', obj.mPtr, uint32(imaging_environment));
    end

    function ret_val = getMaxDistance(obj)
      % getMaxDistance
      %   Get the maximum distance photons reflect from
      %
      % Output:
      %   ret_val - numeric - Maximum distance
      %

      ret_val = tof.mex('tof_user_config_get_max_distance', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setMaxDistance(obj, max_distance)
      % setMaxDistance
      %   Set the maximum distance photons reflect from
      %
      % Inputs:
      %   max_distance - numeric - Maximum distance
      %

      tof.mex('tof_user_config_set_max_distance', obj.mPtr, max_distance);
    end

    function ret_val = getFps(obj)
      % getFps
      %   Get the depth frame rate
      %
      % Output:
      %   ret_val - numeric - The depth fps
      %

      ret_val = tof.mex('tof_user_config_get_fps', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setFps(obj, fps)
      % setFps
      %   Set the depth frame rate
      %
      % Inputs:
      %   fps - numeric - The depth fps
      %

      tof.mex('tof_user_config_set_fps', obj.mPtr, fps);
    end

    function ret_val = getLensFlare(obj)
      % getLensFlare
      %   Get if lens flare is expected in the scene
      %
      % Output:
      %   ret_val - logical - if lens flare is present
      %

      ret_val = tof.mex('tof_user_config_get_lens_flare', obj.mPtr);
    end

    function setLensFlare(obj, lens_flare)
      % setLensFlare
      %   Set if lens flare is expected in the scene
      %
      % Inputs:
      %   lens_flare - logical - if lens flare is present
      %

      tof.mex('tof_user_config_set_lens_flare', obj.mPtr, lens_flare);
    end

    function ret_val = getTranslucent(obj)
      % getTranslucent
      %   Get if there are translucent objects in the scene
      %
      % Output:
      %   ret_val - logical - If translucent objects
      %

      ret_val = tof.mex('tof_user_config_get_translucent', obj.mPtr);
    end

    function setTranslucent(obj, translucent)
      % setTranslucent
      %   Set if there are translucent objects in the scene
      %
      % Inputs:
      %   translucent - logical - If translucent objects
      %

      tof.mex('tof_user_config_set_translucent', obj.mPtr, translucent);
    end

    function ret_val = getHdr(obj)
      % getHdr
      %   Get high dynamic range enabled. Enable this when pixel saturation
      %   occurs. This will only happen for retroreflectors or reflective
      %   objects close (<20cm) to the camera
      %
      % Output:
      %   ret_val - logical - Enable high dynamic range
      %

      ret_val = tof.mex('tof_user_config_get_hdr', obj.mPtr);
    end

    function setHdr(obj, hdr)
      % setHdr
      %   Set high dynamic range enabled. Enable this when pixel saturation
      %   occurs. This will only happen for retroreflectors or reflective
      %   objects close (<20cm) to the camera
      %
      % Inputs:
      %   hdr - logical - Enable high dynamic range
      %

      tof.mex('tof_user_config_set_hdr', obj.mPtr, hdr);
    end

    function ret_val = getIntegrationTime(obj)
      % getIntegrationTime
      %   Get the integration time
      %
      % Output:
      %   ret_val - IntegrationTime - The integration time
      %

      ret_val = tof.mex('tof_user_config_get_integration_time', obj.mPtr);
    end

    function setIntegrationTime(obj, int_time)
      % setIntegrationTime
      %   Set the integration time
      %
      % Inputs:
      %   int_time - IntegrationTime - The integration time
      %

      tof.mex('tof_user_config_set_integration_time', obj.mPtr, uint32(int_time));
    end

    function ret_val = getStrategy(obj)
      % getStrategy
      %   Get what the configuration is being optimized for
      %
      % Output:
      %   ret_val - Strategy - The user configuration strategy
      %

      ret_val = tof.mex('tof_user_config_get_strategy', obj.mPtr);
    end

    function setStrategy(obj, strategy)
      % setStrategy
      %   Set what the configuration is being optimized for
      %
      % Inputs:
      %   strategy - Strategy - The user configuration strategy
      %

      tof.mex('tof_user_config_set_strategy', obj.mPtr, uint32(strategy));
    end

    function ret_val = getChannel(obj)
      % getChannel
      %   Get The camera channel, increment to support multi-camera usage
      %
      % Output:
      %   ret_val - numeric - The channel number, set to 0, otherwise increment
      %                       with number of cameras.
      %

      ret_val = tof.mex('tof_user_config_get_channel', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setChannel(obj, channel)
      % setChannel
      %   Set The camera channel, increment to support multi-camera usage
      %
      % Inputs:
      %   channel - numeric - The channel number, set to 0, otherwise increment
      %                       with number of cameras.
      %

      tof.mex('tof_user_config_set_channel', obj.mPtr, channel);
    end

  end
end
