import ctypes
from .__library__ import lib, get_error


class FrameType:
    """
    The different frame types that are supported
    """

    RAW_COMMON = 0
    "The 16bits coming off the OPT8241 image sensor"

    COMMON = 1
    "Common (A + B) image, 4 or 12 bits depending on the sensor"

    RAW = 2
    "Raw (A - B) image 12 bits signed data"

    REAL = 3
    "The real image"

    IMAGINARY = 4
    "The imaginary image"

    PHASE = 5
    "The phase image"

    AMPLITUDE = 6
    "The amplitude image"

    RADIAL = 7
    "The radial distance image"

    INTENSITY = 8
    "The intensity image"

    X = 9
    "The x image of the point cloud"

    Y = 10
    "The y image of the point cloud"

    Z = 11
    "The z image of the point cloud"

    XYZ = 12
    "A complete pointcloud image: float x, float y, float z, float padding"

    XYZ_AMP = 13
    "A complete pointcloud image with amplitude: float x, float y, float z, float amplitude"

    XYZ_BGR = 14
    "A complete pointcloud image with bgr: float x, float y, float z, uint8 b, uint8 g, uint8 r, uint8 padding"

    XYZ_BGR_I = 15
    "A complete pointcloud image with bgr and intensity: float x, float y, float z, uint8 b, uint8 g, uint8 r, uint8 intensity"

    BGR = 18
    "A color image, encode bgr"

    YUV = 19
    "A color image encoded in YUV422"

    RAW_AB = 20
    "A raw image containing both the A and B tap"

    MJPEG = 21
    "a color image encoded in mjpeg"

    BGR_PROJECTED = 22
    "A color image projected to the XYZ locations of the depth sensor"

    COMPRESSED_DOT = 23
    "The averaged value for each dot. In the XYZAmp format."



class Stream():
    """
    This class describes the different data streams coming from the camera
    """
    def __del__(self):
        if self._ptr:
            lib.tof_stream_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr = None):
        """
        Construct new stream class
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def frameType(self):
        """
        Get the frame type

        Returns
        ----------
        FrameType
            The frame type
        """
        return lib.tof_stream_frame_type(self._ptr, get_error())


    def frameId(self):
        """
        Get the frame id

        Returns
        ----------
        int
            The frame id
        """
        return lib.tof_stream_frame_id(self._ptr, get_error())


    def modulationFrequency(self):
        """
        Get the modulation frequency of the frame

        Returns
        ----------
        float
            The modulation frequency
        """
        return lib.tof_stream_modulation_frequency(self._ptr, get_error())


    def integrationTime(self):
        """
        Get the integration time of the frame

        Returns
        ----------
        int
            The integration time
        """
        return lib.tof_stream_integration_time(self._ptr, get_error())


    def process(self):
        """
        Get a bit array of the processing performed on the frame

        Returns
        ----------
        int
            A bit array
        """
        return lib.tof_stream_process(self._ptr, get_error())


def frameTypeToString(frame_type):
    """
    Get the string representation of the frame type

    Parameters
    ----------
    frame_type : FrameType
        Frame type

    Returns
    ----------
    str
        String representation of frame type
    """
    return lib.tof_frame_type_to_string(frame_type, get_error()).decode("utf-8")


